Package: ggridges
Type: Package
Title: Ridgeline Plots in 'ggplot2'
Version: 0.5.7
Authors@R: 
    person(
      given = "Claus O.",
      family = "Wilke",
      role = c("aut", "cre"),
      email = "wilke@austin.utexas.edu",
      comment = c(ORCID = "0000-0002-7470-9261")
    )
Description: Ridgeline plots provide a convenient way of visualizing changes in distributions over time or space. This package enables the creation of such plots in 'ggplot2'.
URL: https://wilkelab.org/ggridges/
BugReports: https://github.com/wilkelab/ggridges/issues
Depends: R (>= 3.2)
Imports: ggplot2 (>= 3.5.0), grid (>= 3.0.0), scales (>= 0.4.1), withr
        (>= 2.1.1)
License: GPL-2 | file LICENSE
LazyData: true
Suggests: covr, dplyr, patchwork, ggplot2movies, forcats, knitr,
        rmarkdown, testthat, vdiffr
VignetteBuilder: knitr
Collate: 'data.R' 'ggridges.R' 'geoms.R' 'geomsv.R' 'geoms-gradient.R'
        'geom-density-line.R' 'position.R' 'scale-cyclical.R'
        'scale-point.R' 'scale-vline.R' 'stats.R' 'theme.R'
        'utils_ggplot2.R' 'utils.R'
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-08-26 23:53:58 UTC; clauswilke
Author: Claus O. Wilke [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7470-9261>)
Maintainer: Claus O. Wilke <wilke@austin.utexas.edu>
Repository: CRAN
Date/Publication: 2025-08-27 05:10:01 UTC
Built: R 4.6.0; ; 2026-04-28 13:22:35 UTC; unix
