# Generated by roxygen2: do not edit by hand

export(GeomDensityLine)
export(GeomDensityRidges)
export(GeomDensityRidges2)
export(GeomDensityRidgesGradient)
export(GeomRidgeline)
export(GeomRidgelineGradient)
export(GeomVRidgeline)
export(PositionPointsJitter)
export(PositionPointsSina)
export(PositionRaincloud)
export(ScaleCyclical)
export(StatBinline)
export(StatDensityRidges)
export(cyclical_scale)
export(geom_density_line)
export(geom_density_ridges)
export(geom_density_ridges2)
export(geom_density_ridges_gradient)
export(geom_ridgeline)
export(geom_ridgeline_gradient)
export(geom_vridgeline)
export(position_points_jitter)
export(position_points_sina)
export(position_raincloud)
export(scale_alpha_cyclical)
export(scale_color_cyclical)
export(scale_colour_cyclical)
export(scale_fill_cyclical)
export(scale_linetype_cyclical)
export(scale_linewidth_cyclical)
export(scale_point_color_continuous)
export(scale_point_color_discrete)
export(scale_point_color_gradient)
export(scale_point_color_hue)
export(scale_point_colour_continuous)
export(scale_point_colour_discrete)
export(scale_point_colour_gradient)
export(scale_point_colour_hue)
export(scale_point_fill_continuous)
export(scale_point_fill_discrete)
export(scale_point_fill_gradient)
export(scale_point_fill_hue)
export(scale_point_shape)
export(scale_point_shape_discrete)
export(scale_point_size_continuous)
export(scale_size_cyclical)
export(scale_vline_color_continuous)
export(scale_vline_color_discrete)
export(scale_vline_color_gradient)
export(scale_vline_color_hue)
export(scale_vline_colour_continuous)
export(scale_vline_colour_discrete)
export(scale_vline_colour_gradient)
export(scale_vline_colour_hue)
export(scale_vline_linetype)
export(scale_vline_linetype_discrete)
export(scale_vline_width_continuous)
export(stat_binline)
export(stat_density_ridges)
export(theme_ridges)
import(ggplot2)
importFrom(ggplot2,Geom)
importFrom(ggplot2,ScaleDiscrete)
importFrom(ggplot2,Stat)
importFrom(ggplot2,StatBin)
importFrom(ggplot2,draw_key_polygon)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,layer)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(scales,train_discrete)
importFrom(stats,quantile)
