% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geoms.R
\docType{data}
\name{geom_ridgeline}
\alias{geom_ridgeline}
\alias{GeomRidgeline}
\title{Plot a ridgeline (line with filled area underneath)}
\usage{
geom_ridgeline(
  mapping = NULL,
  data = NULL,
  stat = "identity",
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[=aes]{aes()}} or
\code{\link[=aes_]{aes_()}}. If specified and \code{inherit.aes = TRUE} (the
default), it is combined with the default mapping at the top level of the
plot. You must supply \code{mapping} if there is no plot mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[=ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame.}, and
will be used as the layer data.}

\item{stat}{The statistical transformation to use on the data for this
layer, as a string.}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function.}

\item{na.rm}{If \code{FALSE}, the default, missing values are removed with
a warning. If \code{TRUE}, missing values are silently removed.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetics,
rather than combining with them.}

\item{...}{other arguments passed on to \code{\link[=layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{color = "red"} or \code{linewidth = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
Plots the sum of the \code{y} and \code{height} aesthetics versus \code{x}, filling the area between \code{y} and \code{y + height} with a color.
Thus, the data mapped onto y and onto height must be in the same units.
If you want relative scaling of the heights, you can use \code{\link{geom_density_ridges}} with \code{stat = "identity"}.
}
\details{
In addition to drawing ridgelines, this geom can also draw points if they are provided as part of the dataset.
The stat \code{\link[=stat_density_ridges]{stat_density_ridges()}} takes advantage of this option to generate ridgeline plots with overlaid
jittered points.
}
\section{Aesthetics}{


Required aesthetics are in bold.
\itemize{
\item \strong{\code{x}}
\item \strong{\code{y}}
\item \strong{\code{height}} Height of the ridgeline, measured from the respective \code{y} value. Assumed to be positive, though this is not required.
\item \code{group} Defines the grouping. Required when the dataset contains multiple distinct ridgelines. Will typically be the same
variable as is mapped to \code{y}.
\item \code{scale} A scaling factor to scale the height of the ridgelines.
A value of 1 indicates that the heights are taken as is. This aesthetic can be used to convert
\code{height} units into \code{y} units.
\item \code{min_height} A height cutoff on the drawn ridgelines. All values that fall below this cutoff will be removed.
The main purpose of this cutoff is to remove long tails right at the baseline level, but other uses are possible.
The cutoff is applied before any height
scaling is applied via the \code{scale} aesthetic. Default is 0, so negative values are removed.
\item \code{colour} Color of the ridgeline
\item \code{fill} Fill color of the area under the ridgeline
\item \code{alpha} Transparency level of \code{fill}. Not applied to \code{color}. If you want transparent lines, you can set their
color as RGBA value, e.g. #FF0000A0 for partially transparent red.
\item \code{group} Grouping, to draw multiple ridgelines from one dataset
\item \code{linetype} Linetype of the ridgeline
\item \code{linewidth} Line thickness
\item \code{point_shape}, \code{point_colour}, \code{point_size}, \code{point_fill}, \code{point_alpha}, \code{point_stroke} Aesthetics applied
to points drawn in addition to ridgelines.
}
}

\examples{
library(ggplot2)

d <- data.frame(x = rep(1:5, 3), y = c(rep(0, 5), rep(1, 5), rep(3, 5)),
                height = c(0, 1, 3, 4, 0, 1, 2, 3, 5, 4, 0, 5, 4, 4, 1))
ggplot(d, aes(x, y, height = height, group = y)) + geom_ridgeline(fill="lightblue")

}
\keyword{datasets}
